;Low Battery Detection and Data Save Sample 1
;Detects the low battery condition and saves essential data in flash memory

* ------------------------------------------------------
*  Special Function Registers

       .include "sfr.i"

* ------------------------------------------------------
*  Variable-Definitions

;OCR (Oscillation Control Register) settings
;Specifies internal RC oscillation for the system clock (1/12)
osc_rc EQU 77
;Specifies internal RC oscillation for the system clock (1/6)
osc_rcfw EQU 205
;Specifies crystal oscillation for the system clock
osc_xt EQU 239

;Low battery detection flag (RAM bank 0)
LowBattChk EQU 110

;Flash memory write end detection method
fmflag EQU 124
;Flash memory bank switching
fmbank EQU 125
;Flash memory upper address
fmadd_h EQU 126
;Flash memory lower address
fmadd_l EQU 127

;Start of buffer for flash memory reading/writing
fmbuff EQU 128

* ------------------------------------------------------
*  Interupt Vectors

	.include "libraries/f_intvec.i"

* ------------------------------------------------------
*  Game Data

	.include "data/gamehdr.i"
	.include "data/iconanm.i"
	.include "data/iconplt.i"
	.include "data/icondat.i"

* ------------------------------------------------------
*  Main Program Loop

	.include "libraries/main.i"

* ------------------------------------------------------
*  Jump To User Program

	jmp begin

* ------------------------------------------------------
*  User Defined Imports


* ------------------------------------------------------
*  User Defined Program

begin:
	;Clears the LCD display image
	call cls

loop0:
	;Application main processing
	;[M] (mode) Button Check
	ld P3
	;If the [M] button is pressed, the application ends
	bn acc, 6, .finish
	;Battery Status Check
	;Checks the battery status
	call ChkBatt
	;If acc = 0 then battery normal; loops
	bz loop0
	;Low battery processing
	;Prepares data for test save
	;In an actual application, this routine would gather the data that is to be saved and then place the data in the flash ROM write buffer.
	call prepare
	;Writes the data that was prepared in the buffer (to be saved) to flash memory
	call WriteData
;End program
.finish:
	call finish

;Preparing Data for Test Save
;Prepares 128 bytes of data from 10h to 8fh in fmbuff
prepare:
	;Pushes the PSW value onto the stack
	push PSW
	;Selects data RAM bank 1
	set1 PSW, 1

	;Moves the read/write buffer address to r0
	mov #fmbuff, 0
	;Loop counter (128 times)
	mov #128, c
	;Initial value of data to be written
	mov #$10, b

loop4:
	;Places the data in the buffer
	ld b
	st @r0
	;Changes the writing test data
	inc b
	;Increments the buffer address
	inc 0
	;Decrements the loop counter
	dec c
	ld c
	;Repeats 128 times
	bnz loop4
	;Pops the PSW value off of the stack
	pop PSW
	;prepare end
	ret

;Detecting low battery status
;Outputs:
;	acc = 0 : Battery status normal
;	acc = 0ffh: Low battery	
ChkBatt:
	;Checks the status of P71
	ld P7
	;Branches if there is no battery
	bn acc, 1, next3
	;Battery exists
	;acc = 0
	mov #0, acc
	;ChkBatt end. acc = 0 is returned if battery exists
	ret

next3:
	;No battery
	;acc = 0
	mov #$FF, acc
	;ChkBatt end. acc = 0ffh is returned if battery exists
	ret

;Writing Buffer Data to Flash Memory
WriteData:
	;Writing to Flash Memory
	;Pushes the PSW value onto the stack
	push PSW
	;Selects data RAM bank 1
	set1 PSW, 1

	;Flash memory bank specification = 0
	mov #0, fmbank
	;Writing destination address (upper)
	mov #>fmarea, fmadd_h
	;Writing destination address (lower)
	mov #<fmarea, fmadd_l
	;Detects end by toggle bit method
	mov #0, fmflag

	;Selects data RAM bank 0
	clr1 PSW, 1
	;Does not detect low battery (0ffh)
	mov #$FF, acc
	;Low battery automatic detection flag (RAM bank 0)
	st LowBattChk

	; Pops the PSW value off of the stack
	pop PSW
	;Pushes the OCR value onto the stack
	push OCR
	;Specifies the system clock (RC)
	mov #osc_rc, OCR
	;BIOS "Writing to flash memory"
	call fm_wrt_ex
	;Pops the OCR value off of the stack
	pop OCR

	;Pushes the PSW value onto the stack
	push PSW
	;Selects data RAM bank 0
	clr1 PSW, 1
	;Detects low battery (0)
	mov #0, acc
	;Low battery automatic detection flag (RAM bank 0)
	st LowBattChk
	;Pops the PSW value off of the stack
	pop PSW
	;WriteData end
	ret

finish:
	jmpf goodbye

;Clears the LCD display image
cls:
	;Pushes the OCR value onto the stack
	push OCR
	;Specifies the system clock
	mov #osc_rc, OCR
	;Specifies the display RAM bank address (BANK0)
	mov #0, XBNK
	;Clears the data in that bank
	call cls_s
	;Specifies the display RAM bank address (BANK1)
	mov #1, XBNK
	;Clears the data in that bank
	call cls_s
	;Pops the OCR value onto the stack
	pop OCR
	;Cls end
	ret

;Clearing one bank of display RAM
cls_s:
	;Points the indirect addressing register at the start of display RAM
	mov #$80, 2
	;Sets the number of loops in loop counter b
	mov #$80, b
loop3:
	;Writes "0" while incrementing the address
	mov #0, @r2
	inc 2
	;Repeats until b is "0"
	dbnz b, loop3
	;Cls_s end
	ret

;Flash memory area for saving data
;((*-1) land 0ff80h) + 80h
;Aligns with 128-byte boundary
fmarea:
	.byte 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0
	.byte 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0
	.byte 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0
	.byte 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0
	.byte 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0
	.byte 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0
	.byte 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0
	.byte 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0

* ------------------------------------------------------
*  End Program

	.include "libraries/endprog.i"
