;-------------------------------------------------------------------------------
; Useful definitions
;-------------------------------------------------------------------------------

LCD_CHUNK_SELECTOR EQU 2

; The following are graphics subroutines.

* ------------------------------------------------------
*  Clear Screen

clrscr:
       clr1 ocr,5
       ;
       push acc
       push xbnk
       push 2
       ;
       mov #0,xbnk
.cbank:
       mov #$80,2
.cloop:
       mov #0,@R2
       ;
       inc 2
       ;
       ld 2
       and #$f
       bne #$c,.cskip
       ;
       ld 2
       add #4
       st 2
.cskip:
       ld 2
       bnz .cloop
       ;
       bp xbnk,0,.cexit
       ;
       mov #1,xbnk
       br .cbank
.cexit:
       pop 2
       pop xbnk
       pop acc
       set1 ocr,5
       ret


; Explanation:
;
;        clr1 ocr,5
;
; The clr1 instruction clears the fifth bit in the Oscillation Control
; Register.  The previous hexadecimal value was A1, which is the binary
; number 10100001.  The CPU recognizes the right most significant bit as
; the zeroeth bit; the new number in the OCR is 10000001, or 81 in
; hexadecimal.  This new value changes the clock speed to 5 Mhz, the
; appropriate speed for accessing the LCD Frame Buffer of the VMU.
;
;        push acc
;        push xbnk
;        push 2
;
; The values in the accumulator, the bank address register, and memory
; address 002 are stored on the stack.
;
;        mov #0,xbnk
;
; Placing the value 0 into the Bank Address Register sets it to zero.  The
; LCD Frame Buffer is only large enough for half of the LCD screen; bank zero
; lets us work with the top half of the LCD screen.
;
; .cbank:
;        mov #$80,2
;
; The LCD frame buffer starts at address 0180.  So the starting value $80 is
; placed into memory address 2.
;
; .cloop:
;        mov #0,@R2
;
; @R2 is used for indirect addressing and serves as a pointer into the Special
; Function Register Area of the VMU (of which locations $80 - $fb represent
; the LCD Frame Buffer (XRAM) - @R0 and @R1 always point to the General
; Purpose RAM (from 0000 - 00ff), while @R2 and @R3 always point to the
; Special Function Register area (from 0100 - 01ff).
;
; During the first pass, memory address two holds the value $80.  Therefore,
; 00000000 is placed into memory address 0180.  We've now cleared the first 8
; pixels on the LCD screen.
;
;        inc 2
;
; We increment the value of memory address 002.  During the first pass, this
; would change it to $81.
;
;        ld 2
;
; Now we put the value of memory address 002 into the accumulator.  Again, in
; the first pass this is $81.
;
;        and #$f
;
; The CPU performs a bitwise AND with the value in the accumulator and $0F.
; During the first pass, we're comparing 10000001 and 00001111; the new value
; is 00000001.
;
;        bne #$c,.cskip
;
; This instruction is a conditional statement (an "if").  If the value of the
; accumulator is equal to $0C (12 decimal), goto (branch to) subfunction
; ".cskip".  We do this to see if the LCD frame buffer address is divisable
; by 12.  Each six bytes (6 * 8 bits) in the LCD Frame Buffer represent one
; line on the LDC screen (48 pixel), and after every two lines there is a
; space of 4 unused bytes.
;
;        ld 2
;        add #4
;        st 2
;
; If we have written two full lines to the the LCD Frame Buffer (of which the
; current write address is represented in memory address 002), then we add 4,
; to skip the unused bytes and set the pointer to the beginning of the next
; line.  During the first pass it isn't, so this part is skipped.
;
; .cskip:
;        ld 2
;        bnz .cloop
;
; This is a check to see if we're finished with the LCD Frame Buffer in
; bank 0 (half of the screen).  If we've reached the end and roll over to 00,
; go back to ".cloop".
;
;        bp xbnk,0,.cexit
;
; If the Bank Address Register does not equal zero (meaning that we have
; already written to the LCD Frame Buffer in both banks), goto ".cexit".
;
;        mov #1,xbnk
;        br .cbank
;
; Move 1 into the Bank Address Register (switch to bank 1).  This lets us
; clear the bottom half of the screen.  We then continue with writing to the
; lower half of the screen just like we previously wrote to the upper half
; (because all that's different is that the lower half is in another bank).
;
; .cexit:
;        pop 2
;        pop xbnk
;        pop acc
;        set1 ocr,5
;        ret
;
; This restores the values of memory address 2, the Bank Address Register,
; and the Accumulator.  The clock speed is reset to A1 and the function
; returns to the address following the code that called it.


* ------------------------------------------------------
*  Set Screen

setscr:
       clr1 ocr,5
       push acc
       push xbnk
       ;
       push c
       push 2
       mov #$80,2
       ;
       xor acc
       st xbnk
       st c
.sloop:
       ldc
       st @R2
       ;
       inc 2
       ;
       ld 2
       and #$f
       bne #$c,.sskip
       ;
       ld 2
       add #4
       st 2
       ;
       bnz .sskip
       ;
       inc xbnk
       mov #$80,2
.sskip:
       inc c
       ld c
       bne #$c0,.sloop
       ;
       pop 2
       pop c
       pop xbnk
       pop acc
       set1 ocr,5
       ret


; Explanation:
;
;        clr1 ocr,5
;        push acc
;        push xbnk
;
; Setscr is similar to clrscr.  The C register is used to as a counter for
; the table data (the picture we're going to copy to the LCD screen).
;
;        push c
;        push 2
;        mov #$80,2
;
;        xor acc
;        st xbnk
;        st c
;
; By XORing the accumulator with itself, we are setting it to zero.
;
; .sloop:
;        ldc
;
; LDC creates a 16-bit address by combining the 16-bit address in TRH/TRL
; with the value in the accumulator.  The byte at this address (the beginning
; of our picture data) is then loaded into the accumulator.
;
; In the first pass, ACC contains the value zero, so the value at address
; (TRH/TRL + 0) is loaded.
;
;        st @R2
;
; We then write this data into the LCD Frame Buffer.
;
;        inc 2
;
;        ld 2
;        and #$f
;        bne #$c,.sskip
;
;        ld 2
;        add #4
;        st 2
;
;        bnz .sskip
;
;        inc xbnk
;        mov #$80,2
;
; All of the above is similar to clrscr.
;
; .sskip:
;        inc c
;        ld c
;
; Here we increase C by one, and load the new value into the accumulator.
;
;        bne #$c0,.sloop
;
; If the accumulator has not yet reached hexadecimal value C0 (192 decimal,
; 6 bytes per screenline * 32 lines), we branch back to ".sloop".  Since the
; accumulator now contains a value that has been increased by one, during this
; pass we will now load the byte from our screen-data that follows the one we
; loaded during our previous pass.
;
;        pop 2
;        pop c
;        pop xbnk
;        pop acc
;        set1 ocr,5
;        ret
;
; Again the previously saved values are restored and the function returns to
; the address following the code that called it.

;-------------------------------------------------------------------------------
; Plot a pixel
; Input:
;	b - row
;   c - column
;-------------------------------------------------------------------------------
put_pixel:
	push b
	push c
	push acc
	mov #0, xbnk			; bank 0 for rows 0-15, bank 1 for rows 16-31
	
	ld b
	clr1 psw,cy
	rorc
	clr1 psw,cy
	rorc
	clr1 psw,cy
	rorc
	clr1 psw,cy
	rorc
	clr1 psw,cy
	bz putpixel_bank_ok		; if( b div 16 == 0 ) then goto putpixel_bank_ok

	mov #1, xbnk 			; since row >= 16, we use bank 1
	ld b
	sub #16
	st b					; b := b - 16
	
putpixel_bank_ok:
	push b
	push c
	push c
	push b
	clr1 ocr, 5

	; screen resolution is 48x32
	; each chunk has 8 pixels
	; there are 6*32 chunks (6 on each line, 32 lines)
	
	; multiply row by 6 to get the number of the first chunk in the row
	; where we will draw the pixel
	mov #0, acc
	mov #6, c
	mul						; [b:acc:c] = [acc:c] * b
	
	; since row is between 0 and 31, the result of multiplication by 6
	; only requires the lower byte, which ends up in register c after the
	; multiplication
	
	pop acc					; acc := b
	
	; the LCD_CHUNK_SELECTOR maps chunk number to LCD like so:
	;  0  1  2  3  4  5       <---- first LCD line (6 * 8 = 48 pixels)
	;  6  7  8  9  A  B       <---- second LCD line
	; 10 11 12 13 14 15       <---- third line
	; 16 17 18 19 1A 1B       <---- fourth line
	;                               and so on
	; It's important to note that last 4 of every 16 chunks are offscreen 
	
	; now, for every two rows, we must add 4 to the chunk number, 
	; to account for the 4 offscreen chunks					; 
	clr1 psw,cy
	rorc					; acc := b div 2
	clr1 psw,cy				; clear CY flag to discard rotated bit
	rolc					;
	clr1 psw,cy
	rolc					; acc := acc * 4 (the offscreen adjustment)
	clr1 psw,cy
	add c					; acc := (row * 6) + offscreen adjustment
							; (basically contains the number of first chunk
							; on the row where we'll put the pixel )
	
	pop c					; c now contains the column argument
	push acc
	; now calculate the correct chunk on this row where we will put the pixel
	mov #0, acc				
	mov #8, b				
	div						; [acc:c] := [acc:c] div b
							; b := [acc:c] mod b
	
	; b now contains a value between 0 and 7
	; corresponding to the bit which has to be set in the byte we'll write to LCD
	pop acc
	add c					; acc now contains correct chunk to set
	
	add #$80				; 0x80 is the first chunk (top left of screen)
							; 0x81 the second (next 8 pixels to the right), and so on
	st LCD_CHUNK_SELECTOR
	
	; set the correct bit for our pixel, in a byte which will be written to LCD
	; we do this by shifting a 1 as many places as the value in register b
	mov #%10000000, acc
putpixel_shift:
	push acc
	ld b
	bz putpixel_shift_done	; if b == 0 then we are done shifting our bit
	pop acc
	clr1 psw,cy
	rorc					
	clr1 psw,cy
	
	dec b
	br putpixel_shift
putpixel_shift_done:
	pop acc					; we missed the pop right after the bz, so do it here
	; acc now contains the proper byte we have to write to get our pixel

	or @R2					; this accumulates pixels, so that the zeroes we
							; write along with the 1 don't erase existing pixels
	
	st @R2					; write accumulated (on-screen + our new pixel) pixels
							; to the LCD
	;set1 ocr,5
	
	pop c
	pop b
	
	pop acc
	pop c
	pop b
	ret

;-------------------------------------------------------------------------------
; Clear a pixel
; Input:
;	b - row
;   c - column
;-------------------------------------------------------------------------------
clear_pixel:
	push b
	push c
	push acc
	mov #0, xbnk			; bank 0 for rows 0-15, bank 1 for rows 16-31
	
	ld b
	clr1 psw,cy
	rorc
	clr1 psw,cy
	rorc
	clr1 psw,cy
	rorc
	clr1 psw,cy
	rorc
	clr1 psw,cy
	bz clearpixel_bank_ok		; if( b div 16 == 0 ) then goto clearpixel_bank_ok

	mov #1, xbnk 			; since row >= 16, we use bank 1
	ld b
	sub #16
	st b					; b := b - 16
	
clearpixel_bank_ok:
	push b
	push c
	push c
	push b
	clr1 ocr, 5

	; screen resolution is 48x32
	; each chunk has 8 pixels
	; there are 6*32 chunks (6 on each line, 32 lines)
	
	; multiply row by 6 to get the number of the first chunk in the row
	; where we will draw the pixel
	mov #0, acc
	mov #6, c
	mul						; [b:acc:c] = [acc:c] * b
	
	; since row is between 0 and 31, the result of multiplication by 6
	; only requires the lower byte, which ends up in register c after the
	; multiplication
	
	pop acc					; acc := b
	
	; the LCD_CHUNK_SELECTOR maps chunk number to LCD like so:
	;  0  1  2  3  4  5       <---- first LCD line (6 * 8 = 48 pixels)
	;  6  7  8  9  A  B       <---- second LCD line
	; 10 11 12 13 14 15       <---- third line
	; 16 17 18 19 1A 1B       <---- fourth line
	;                               and so on
	; It's important to note that last 4 of every 16 chunks are offscreen 
	
	; now, for every two rows, we must add 4 to the chunk number, 
	; to account for the 4 offscreen chunks					; 
	clr1 psw,cy
	rorc					; acc := b div 2
	clr1 psw,cy				; clear CY flag to discard rotated bit
	rolc					;
	clr1 psw,cy
	rolc					; acc := acc * 4 (the offscreen adjustment)
	clr1 psw,cy
	add c					; acc := (row * 6) + offscreen adjustment
							; (basically contains the number of first chunk
							; on the row where we'll put the pixel )
	
	pop c					; c now contains the column argument
	push acc
	; now calculate the correct chunk on this row where we will put the pixel
	mov #0, acc				
	mov #8, b				
	div						; [acc:c] := [acc:c] div b
							; b := [acc:c] mod b
	
	; b now contains a value between 0 and 7
	; corresponding to the bit which has to be set in the byte we'll write to LCD
	pop acc
	add c					; acc now contains correct chunk to set
	
	add #$80				; 0x80 is the first chunk (top left of screen)
							; 0x81 the second (next 8 pixels to the right), and so on
	st LCD_CHUNK_SELECTOR
	
	; set the correct bit for our pixel, in a byte which will be written to LCD
	; we do this by shifting a 1 as many places as the value in register b
	mov #%10000000, acc
clearpixel_shift:
	push acc
	ld b
	bz clearpixel_shift_done	; if b == 0 then we are done shifting our bit
	pop acc
	clr1 psw,cy
	rorc					
	clr1 psw,cy
	
	dec b
	br clearpixel_shift
clearpixel_shift_done:
	pop acc					; we missed the pop right after the bz, so do it here
	; acc now has only one bit set, in the position we want to clear
	; we have to invert acc now, to make it all but one zeroes
	not1 acc, 0
	not1 acc, 1
	not1 acc, 2
	not1 acc, 3
	not1 acc, 4
	not1 acc, 5
	not1 acc, 6
	not1 acc, 7
	
	; acc now contains the proper byte we have to write to clear our pixel
	
	and @R2					; we and with the existing pixels to keep all but the one
	
	st @R2					; write accumulated (on-screen + our new pixel) pixels
							; to the LCD
	;set1 ocr,5
	
	pop c
	pop b
	
	pop acc
	pop c
	pop b
	ret

;-------------------------------------------------------------------------------
; Read a pixel
; Input:
;	b - row
;   c - column
; Output:
;	zero in acc when pixel is not set
;   non-zero in acc when pixel is set
;-------------------------------------------------------------------------------
read_pixel:
	push b
	push c
	push acc
	mov #0, xbnk			; bank 0 for rows 0-15, bank 1 for rows 16-31
	
	ld b
	clr1 psw,cy
	rorc
	clr1 psw,cy
	rorc
	clr1 psw,cy
	rorc
	clr1 psw,cy
	rorc
	clr1 psw,cy
	bz readpixel_bank_ok		; if( b div 16 == 0 ) then goto readpixel_bank_ok

	mov #1, xbnk 			; since row >= 16, we use bank 1
	ld b
	sub #16
	st b					; b := b - 16
	
readpixel_bank_ok:
	push b
	push c
	push c
	push b
	clr1 ocr, 5

	; screen resolution is 48x32
	; each chunk has 8 pixels
	; there are 6*32 chunks (6 on each line, 32 lines)
	
	; multiply row by 6 to get the number of the first chunk in the row
	; where we will draw the pixel
	mov #0, acc
	mov #6, c
	mul						; [b:acc:c] = [acc:c] * b
	
	; since row is between 0 and 31, the result of multiplication by 6
	; only requires the lower byte, which ends up in register c after the
	; multiplication
	
	pop acc					; acc := b
	
	; the LCD_CHUNK_SELECTOR maps chunk number to LCD like so:
	;  0  1  2  3  4  5       <---- first LCD line (6 * 8 = 48 pixels)
	;  6  7  8  9  A  B       <---- second LCD line
	; 10 11 12 13 14 15       <---- third line
	; 16 17 18 19 1A 1B       <---- fourth line
	;                               and so on
	; It's important to note that last 4 of every 16 chunks are offscreen 
	
	; now, for every two rows, we must add 4 to the chunk number, 
	; to account for the 4 offscreen chunks					; 
	clr1 psw,cy
	rorc					; acc := b div 2
	clr1 psw,cy				; clear CY flag to discard rotated bit
	rolc					;
	clr1 psw,cy
	rolc					; acc := acc * 4 (the offscreen adjustment)
	clr1 psw,cy
	add c					; acc := (row * 6) + offscreen adjustment
							; (basically contains the number of first chunk
							; on the row where we'll put the pixel )
	
	pop c					; c now contains the column argument
	push acc
	; now calculate the correct chunk on this row where we will put the pixel
	mov #0, acc				
	mov #8, b				
	div						; [acc:c] := [acc:c] div b
							; b := [acc:c] mod b
	
	; b now contains a value between 0 and 7
	; corresponding to the bit which has to be set in the byte we'll write to LCD
	pop acc
	add c					; acc now contains correct chunk to set
	
	add #$80				; 0x80 is the first chunk (top left of screen)
							; 0x81 the second (next 8 pixels to the right), and so on
	st LCD_CHUNK_SELECTOR
	
	; set the correct bit for our pixel, in a byte which will be written to LCD
	; we do this by shifting a 1 as many places as the value in register b
	mov #%10000000, acc
readpixel_shift:
	push acc
	ld b
	bz readpixel_shift_done	; if b == 0 then we are done shifting our bit
	pop acc
	clr1 psw,cy
	rorc					
	clr1 psw,cy
	
	dec b
	br readpixel_shift
readpixel_shift_done:
	pop acc					; we missed the pop right after the bz, so do it here	
	; acc now contains the proper byte with which we can AND and read the pixel
	
	and @R2					; acc becomes non-zero if pixel is set, and zero if not
	
	;set1 ocr,5
	
	pop c
	pop b
	
	pop acc
	pop c
	pop b
	ret
