;Serial Communications Sample 2 (Data Reception)
;Displays a numeric value that was received from the serial communications port on the LCD

* ------------------------------------------------------
*  Special Function Registers

       .include "sfr.i"

* ------------------------------------------------------
*  Variable-Definitions

;Oscillation Control Register settings
;Internal RC oscillation for system clock
osc_rc EQU 77
;Crystal oscillation for system clock
osc_xt EQU 239

;;Low battery detection flag (RAM bank 0)
LowBattChk EQU 110

;Counter
counter = $30
;Work (used in put2digit)
work1 = $31

* ------------------------------------------------------
*  Interupt Vectors

	.include "libraries/intvec.i"

* ------------------------------------------------------
*  Game Data

	.include "data/gamehdr.i"
	.include "data/iconanm.i"
	.include "data/iconplt.i"
	.include "data/icondat.i"

* ------------------------------------------------------
*  Main Program Loop

	.include "libraries/main.i"

* ------------------------------------------------------
*  Jump To User Program

	;jmp begin

* ------------------------------------------------------
*  User Defined Imports


* ------------------------------------------------------
*  User Defined Program

begin:
	;Clears the LCD display
	call cls
	;Turns off the low battery automatic detection function
	call BattChkOff

cwait:
	;Serial communications initialization
	call SioInit
	;Starts if VM is connected
	bz startprog
	;[M] button check
	ld P3
	;If the [M] button is pressed, the application ends
	bn acc, 6, .finish
	;Waits until VM is connected
	jmp cwait
.finish:
	call finish

startprog:

loop0:
	;Receives one byte
	call SioRecv1
	;If there is no received data, then goes to next4
	bnz next4
	;Loads the received data into acc
	ld b
	;Display coordinates (horizontal)
	mov #2, c
	;Display coordinates (vertical)
	mov #1, b
	;Displays the two-digit value on the LCD
	call put2digit
next4:
	;[M] (mode) Button Check
	ld P3
	;If the [M] button is pressed, the application ends
	bn acc, 6, .finish
	;Repeat
	jmp loop0
.finish:
	call finish

;End program
finish:
	;Serial communications end processing
	call SioEnd
	;Turns on the low battery automatic detection function
	call BattChkOn
	jmpf goodbye

;Serial Communications Initialization
;Outputs:
;	acc = 0 : Normal end
;	acc = 0ffh: VM not connected

;Serial communications initialization
;This sample assumes that the system clock is in crystal mode.
SioInit:
	;VM Connection Check
	;Checks the connection status
	ld P7
	;Checks P70, P72, P73
	and #%00001101
	;P70 = 0, P72 = 0, P73 = 1
	sub #%00001000
	;To next3 if connected
	bz next3
	;If not connected, abnormal end with acc = 0ffh
	mov #$ff, acc
	;SioInit end
	ret

next3:
	;Serial communications initialization
	;Specifies output as 'LSB first'
	mov #0, SCON0
	;Specifies input as 'LSB first'
	mov #0, SCON1
	;Sets the transfer rate
	mov #$88, SBR
	;Clears the P10 latch (P10/S00)
	clr1 P1, 0
	;Clears the P12 latch (P12/SCK0)
	clr1 P1, 2
	;Clears the P13 latch (P13/S01)
	clr1 P1, 3
	;Sets the pin functions
	mov #%00000101, P1FCR
	;Sets the pin functions
	mov #%00000101, P1DDR
	;Clears the transfer buffer
	mov #0, SBUF0
	;Clears the transfer buffer
	mov #0, SBUF1
	;SioInit end
	ret

;Serial communications end
;Serial communications end processing
SioEnd:
	;SCON0 = 0
	mov #0, SCON0
	;SCON1 = 0
	mov #0, SCON1
	;P1FCR = 0bfh
	mov #$bf, P1FCR
	;P1DDR = 0a4h
	mov #$a4, P1DDR
	;SioEnd end
	ret

;Receiving 1 byte from a serial port
;Outputs:
;	b: Received data
;	acc = 0 : Received data found
;	acc = 0ffh: Received data not found
;Receiving 1 byte
SioRecv1:
	ld SCON1
	;If received data is found, then go to next5
	bp acc, 1, next5
	;If transfer is currently in progress, then go to next6
	bp acc, 3, next6
	;Starts transfer
	set1 SCON1, 3

next6:
	;Returns with acc = 0ffh (received data not found)
	mov #$ff, acc
	;SioRecv1 end
	ret

next5:
	;Loads the received data
	ld SBUF1
	;Copies the data into b
	st b
	;Resets the transfer end flag
	clr1 SCON1, 1
	;Returns with acc = 0 (received data found)
	mov #0, acc
	;SioRecv1 end
	ret

;Displaying a two digit value
;Inputs
;	acc: Numeric value
;	c: Horizontal position of character
;	b: Vertical position of character
put2digit:
	;Pushes the coordinate data onto the stack
	push b
	push c
	;Calculates the tens digit and the ones digit
	st c
	;( acc = acc/10, work1 = acc mod 10 )
	xor acc
	mov #10, b
	div
	ld b
	;Stores the ones digit in work1
	st work1
	ld c
	;Pops the coordinate values into (c, b)
	pop c
	pop b
	;Pushes the coordinates onto the stack again
	push b
	push c
	;Displays the tens digit
	call putch
	;Loads the ones digit
	ld work1
	;Pops the coordinate values into (c, b)
	pop c
	pop b
	;Moves the display coordinates to the right
	inc c
	;Displays the ones digit
	call putch
	;Put2digit end
	ret

;Clears the LCD display image
cls:
	;Pushes the OCR value onto the stack
	push OCR
	;Specifies the system clock
	mov #osc_rc, OCR
	;Specifies the display RAM bank address (BANK0)
	mov #0, XBNK
	;Clears the data in that bank
	call cls_s
	;Specifies the display RAM bank address (BANK1)
	mov #1, XBNK
	;Clears the data in that bank
	call cls_s
	;Pops the OCR value onto the stack
	pop OCR
	;Cls end
	ret

;Clearing one bank of display RAM
cls_s:
	;Points the indirect addressing register at the start of display RAM
	mov #$80, 2
	;Sets the number of loops in loop counter b
	mov #$80, b
loop3:
	;Writes "0" while incrementing the address
	mov #0, @r2
	inc 2
	;Repeats until b is "0"
	dbnz b, loop3
	;Cls_s end
	ret

;Displaying one character at a specified position
;Inputs
;	acc: Character code
;	c: Horizontal position of character
;	b: Vertical position of character
putch:
	push XBNK
	push acc
	;Calculates display RAM address according to coordinates
	call locate
	pop acc
	;Displays one character
	call put_chara
	pop XBNK
	;Putch end
	ret

;Calculating the display RAM address according to the display position specification
;Inputs
;	c: Horizontal position (0 to 5)
;	b: Vertical position (0 to 3)
;Outputs
;	2: RAM address
;	XBNK: display RAM bank
;Determining the display RAM bank address
locate:
	;Jump to next1 when b >= 2
	ld b
	sub #2
	bn PSW, 7, next1
	;Specifies the display RAM bank address (BANK0)
	mov #$0, XBNK
	br next2

next1:
	st b
	;Specifies the display RAM bank address (BANK1)
	mov #$01, XBNK

;Calculating the RAM address for a specified position on the display
next2:
	;b * 40h + c + 80h
	ld b
	rol
	rol
	rol
	rol
	rol
	rol
	add c
	add #$80
	;Stores the RAM address in 2
	st 2
	;Locate end
	ret

put_chara:
	;Pushes the PSW value onto the stack
	push PSW
	;Selects data RAM bank 1
	set1 PSW, 1
	;Calculating the character data address
	;(TRH,TRL) = acc*8 + fontdata
	rol
	rol
	rol
	add #<fontdata
	st TRL
	mov #0, acc
	addc #>fontdata
	st TRH
	;Pushes the OCR value onto the stack
	push OCR
	;Specifies the system clock
	mov #osc_rc, OCR
	;Offset value for loading the character data
	mov #0, b
	;Loop counter
	mov #4, c
loop1:
	;Loads the display data for the first line
	ld b
	ldc
	;Increments the load data offset by 1
	inc b
	;Transfers the display data to display RAM
	st @r2
	ld 2
	;Adds 6 to the display RAM address
	add #6
	st 2

	;Loads the display data for the second line
	ld b
	ldc
	;Increments the load data offset by 1
	inc b
	;Transfers the display data to display RAM
	st @r2
	ld 2
	;Adds 10 to the display RAM address
	add #10
	st 2

	;Decrements the loop counter
	dec c
	ld c
	;Repeats for 8 lines (four times)
	bnz loop1
	;Pops the OCR value off of the stack
	pop OCR
	;Pops the PSW value off of the stack
	pop PSW
	;Put_chara end
	ret

;Character bit image data
fontdata:
	.byte $7c, $e6, $c6, $c6, $c6, $ce, $7c, $00	; 0
	.byte $18, $38, $18, $18, $18, $18, $3c, $00	; 1
	.byte $7c, $c6, $c6, $0c, $38, $60, $fe, $00	; 2
	.byte $7c, $e6, $06, $1c, $06, $e6, $7c, $00	; 3
	.byte $0c, $1c, $3c, $6c, $cc, $fe, $0c, $00	; 4
	.byte $fe, $c0, $fc, $06, $06, $c6, $7c, $00	; 5
	.byte $1c, $30, $60, $fc, $c6, $c6, $7c, $00	; 6
	.byte $fe, $c6, $04, $0c, $18, $18, $38, $00	; 7
	.byte $7c, $c6, $c6, $7c, $c6, $c6, $7c, $00	; 8
	.byte $7c, $c6, $c6, $7e, $06, $0c, $78, $00	; 9

;Low battery automatic detection function ON
BattChkOn:
	;Pushes the PSW value onto the stack
	push PSW
	;Selects data RAM bank 0
	clr1 PSW, 1
	;Detects low battery (0)
	mov #0, acc
	;Low battery automatic detection flag (RAM bank 0)
	st LowBattChk
	;Pops the PSW value off of the stack
	pop PSW
	;BattChkOn end
	ret

;Low battery automatic detection function OFF
BattChkOff:
	;Pushes the PSW value onto the stack
	push PSW
	;Selects data RAM bank 0
	clr1 PSW, 1
	;Does not detect low battery (0ffh)
	mov #$ff, acc
	;Low battery automatic detection flag (RAM bank 0)
	st LowBattChk
	;Pops the PSW value off of the stack
	pop PSW
	;BattChkOff end
	ret

* ------------------------------------------------------
*  End Program

	.include "libraries/endprog.i"

