#! /bin/sh

# Stop script on error
set -e

is_root_user()   { [ $(id -u) -eq 0 ]; }
program_exists() { command -v "$1" >/dev/null 2>&1; }

#Variables for directories
SCRIPT_DIR=$PWD
ARCHIVE_DIR="$SCRIPT_DIR/../archive"
EXAMPLE_DIR="$SCRIPT_DIR/../examples"
PATCH_DIR="$SCRIPT_DIR/../patchfiles"

INSTALL_DIR="/opt/toolchains/vms"
TEMP_DIR="$INSTALL_DIR/temp"

is_root_user && { echo "Please don't run this script as root."; exit 1; }

if program_exists getconf; then
	CORE_COUNT="$MAKEFLAGS -j$(getconf _NPROCESSORS_ONLN)"
else
	CORE_COUNT=2
fi

if ! program_exists sudo; then
	echo "Please install sudo."
	exit 1
fi

# Install required programs on debian or derived distributions
if program_exists apt-get; then	
	sudo apt-get -y install libx11-dev gcc make tar
elif program_exists yum && program_exists sudo; then	
	sudo yum -y install libx11-dev gcc make tar
# Catch-all, installs requirements when no package manager is available
else
	echo "apt-get nor yum not installed; Case not handled!"	
fi

# Check for programs
MISSING_PROGRAMS=0
for i in gcc make tar; do
	program_exists "$i" || { echo "Please install $i."; MISSING_PROGRAMS=1; }
done

[ $MISSING_PROGRAMS -eq 1 ] && exit 1

if program_exists sudo; then
	if [ -d "$INSTALL_DIR" ]; then
		echo "Install directory exists, deleting old sources"		
		sudo rm -r $INSTALL_DIR	
	fi		
		
	sudo mkdir -p    "$INSTALL_DIR"
	sudo chmod o+rwx "$INSTALL_DIR"
	
	sudo mkdir -p    "$TEMP_DIR"
	sudo chmod o+rwx "$TEMP_DIR"	
else
	if [ -d "$INSTALL_DIR" ]; then
		echo "Install directory exists, deleting old sources"		
		sudo rm -r $INSTALL_DIR	
	fi	
	
	mkdir -p "$INSTALL_DIR"
	mkdir -p "$TEMP_DIR"
fi

#Unpack tar file for SoftVMS
echo "Setting up SoftVMS 1.9"

tar -xvf "$ARCHIVE_DIR/softvms.tar.gz" -C $TEMP_DIR

#Run configure to ready Linux install make files, then make

cd "$TEMP_DIR/vms"

echo "Patching SoftVMS to allow saving of data"

./configure
cp -fv "$PATCH_DIR/softvms/cpu.c" "$TEMP_DIR/vms"
cp -fv "$PATCH_DIR/softvms/mainwin.c" "$TEMP_DIR/vms"
make

echo "SoftVMS patched and compiled!"

#Unpack tar file for VMS assembler

echo "Setting up aslc86k assembler"

tar -xvf "$ARCHIVE_DIR/aslc86k.tar.gz" -C $TEMP_DIR

#No need to patch, this compiles as is!

cd "$TEMP_DIR/as"

make

echo "aslc86k patched and compiled!"

#Unpack tar file for VMS disassembler

echo "Setting up LC86104C/108C disassembler"

tar -xvf "$ARCHIVE_DIR/lcdis.tar.gz" -C $TEMP_DIR

#Need to change references for stricmp and strnicmp to Linux equivalents

cd "$TEMP_DIR/lcdis"

echo "Patching lcdis to define aliases for stricmp and strnicmp to their platform equivalents"

cp -fv "$PATCH_DIR/lcdis/lcdis.c" "$TEMP_DIR/lcdis"
make lcdis.c lcdis

echo "LC86104C/108C disassembler patched and compiled!"

#Copying compiled sources and examples to install directory

echo "Copying compiled files to install directory!"

echo "Copying SoftVMS"

cp -fv "$TEMP_DIR/vms/vms" "$INSTALL_DIR"

echo "Copying aslc86k"

cp -fv "$TEMP_DIR/as/aslc86k" "$INSTALL_DIR"

echo "Copying lcdis"

cp -fv "$TEMP_DIR/lcdis/lcdis" "$INSTALL_DIR"

echo "Copying example files"

cp -rfv "$EXAMPLE_DIR" "$INSTALL_DIR"

echo "Copying include file header sfr.i for aslc86k"

cp -fv "$ARCHIVE_DIR/sfr.i" "$INSTALL_DIR"

echo "Copying VMS documentation"

cp -fv "$ARCHIVE_DIR/VMU.pdf" "$INSTALL_DIR"

echo "Copying README"

cp -fv "$SCRIPT_DIR/../README" "$INSTALL_DIR"

#Cleaning up temp files
echo "Cleaning up temp files"

sudo rm -r $TEMP_DIR

#FINISHED!
echo "\n"
echo "Finished compilation of VMS dev tools."
echo "Please view included example files on how to successfully write a VMS program"
echo "Special thanks to:"
echo "Marcus Comstedt for writing SoftVMS, the VMS assembler and header file, and the help writing this script!"
echo "Tyro99 for the modifications to SoftVMS, Chao Editor 1 and 2, and additional support"
echo "John Maushammer for writing the lcdis VMS disassembler"
echo "\n"
echo "Please visit http://mc.pp.se/dc and http://www.deco.franken.de/myfiles/myfiles.html for additional sources and information on writing VMS programs"
